﻿using ProjectBattleship.CollectionGenericObjects;
using ProjectBattleship.DrawingObject;
using ProjectBattleship.Exceptions;

namespace Battleship.CollectionGenericObjects;

/// <summary>
/// Класс-хранилище коллекций
/// </summary>
/// <typeparam name="T"></typeparam>
public class StorageCollection<T> where T : DrawingWarship
{
    /// <summary>
    /// Словарь (хранилище) с коллекциями
    /// </summary>
    private Dictionary<string, ICollectionGenericObjects<T>> _storages;
    /// <summary>
    /// Возвращение списка названий коллекций
    /// </summary>
    public List<string> Keys => _storages.Keys.ToList();

    /// <summary>
    /// Ключевое слово, с которого должен начинаться файл
    /// </summary>
    private readonly string _collectionKey = "CollectionsStorage";

    /// <summary>
    /// Разделитель для записи ключа и значения элемента словаря
    /// </summary>
    private readonly string _separatorForKeyValue = "|";

    /// <summary>
    /// Разделитель для записей коллекции данных в файл
    /// </summary>
    private readonly string _separatorItems = ";";

    /// <summary>
    /// Конструктор
    /// </summary>
    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
    }
    /// <summary>
    /// Добавление коллекции в хранилище
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <param name="collectionType">тип коллекции</param>
	public void AddCollection(string name, CollectionType collectionType)
    {
        if (string.IsNullOrEmpty(name) || _storages.ContainsKey(name))
            return;
        switch (collectionType)
        {
            case CollectionType.List:
                _storages.Add(name, new ListGenericObjects<T>());
                break;
            case CollectionType.Massive:
                _storages.Add(name, new MassiveGenericObjects<T>());
                break;
            default:
                break;
        }
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    public void DelCollection(string name)
    {
        if (_storages.ContainsKey(name)) { _storages.Remove(name); }
    }
    /// <summary>
    /// Доступ к коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <returns></returns>
	public ICollectionGenericObjects<T>? this[string name]
    {
        get
        {
            if (_storages.TryGetValue(name, out ICollectionGenericObjects<T>? value))
                return value;
            return null;
        }
    }
    /// <summary>
    /// Сохранение информации по самолетам в хранилище в файл
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
    public void SaveData(string filename)
    {
        if (_storages.Count == 0)
        {
            throw new Exception("В хранилище отсутствуют коллекции для сохранения");
        }


        if (File.Exists(filename))
        {
            File.Delete(filename);
        }

        using FileStream fs = new(filename, FileMode.Create);
        using StreamWriter streamWriter = new StreamWriter(fs);
        streamWriter.Write(_collectionKey);

        foreach (KeyValuePair<string, ICollectionGenericObjects<T>> value in _storages)
        {
            streamWriter.Write(Environment.NewLine);

            if (value.Value.Count == 0)
            {
                continue;
            }

            streamWriter.Write(value.Key);
            streamWriter.Write(_separatorForKeyValue);
            streamWriter.Write(value.Value.GetCollectionType);
            streamWriter.Write(_separatorForKeyValue);
            streamWriter.Write(value.Value.MaxCount);
            streamWriter.Write(_separatorForKeyValue);


            foreach (T? item in value.Value.GetItems())
            {
                string data = item?.GetDataForSave() ?? string.Empty;
                if (string.IsNullOrEmpty(data))
                {
                    continue;
                }


                streamWriter.Write(data);
                streamWriter.Write(_separatorItems);

            }
        }
    }
    /// <summary>
    /// Загрузка информации по кораблям в хранилище из файла
    /// </summary>
    /// <param name="filename"></param>
    public void LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            throw new FileNotFoundException("Файл не существует");
        }

        using (StreamReader sr = new StreamReader(filename))
        {
            string? str;
            str = sr.ReadLine();
            if (str != _collectionKey.ToString())
                throw new FormatException("В файле неверные данные");
            _storages.Clear();
            while ((str = sr.ReadLine()) != null)
            {
                string[] record = str.Split(_separatorForKeyValue);
                if (record.Length != 4)
                {
                    continue;
                }
                CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
                ICollectionGenericObjects<T>? collection = StorageCollection<T>.CreateCollection(collectionType);
                if (collection == null)
                {
                    throw new InvalidOperationException("Не удалось определить тип коллекции:" + record[1]);
                }

                collection.MaxCount = Convert.ToInt32(record[2]);

                string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    if (elem?.CreateDrawingWarship() is T Warship)
                    {
                        try
                        {
                            if (collection.Insert(Warship) == -1)
                            {
                                throw new InvalidOperationException("Объект не удалось добавить в коллекцию: " + record[3]);
                            }
                        }
                        catch (CollectionOverflowException ex)
                        {
                            throw new CollectionOverflowException("Коллекция переполнена", ex);
                        }
                    }
                }
                _storages.Add(record[0], collection);
            }
        }
    }
    private static ICollectionGenericObjects<T>? CreateCollection(CollectionType collectionType) => collectionType switch
    {
        CollectionType.Massive => new MassiveGenericObjects<T>(),
        CollectionType.List => new ListGenericObjects<T>(),
        _ => null,
    };
}