﻿using ProjectBattleship.Entities;
namespace ProjectBattleship.DrawingObject;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение базового объекта-сущности
/// </summary>
public class DrawingWarship
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityWarship? EntityWarship { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки военного корабля
    /// </summary>
    protected int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки военного корабля
    /// </summary>
    protected int? _startPosY;
    private EntityWarship warship;

    /// <summary>
    /// Ширина прорисовки военного корабля
    /// </summary>
    private readonly int _drawingWarshipWidth = 150;
    /// <summary>
    /// Высота прорисовки военного корабля
    /// </summary>
    private readonly int _drawingWarshipHeight = 50;
    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawingWarship()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawingWarship(int speed, double weight, Color bodyColor) : this()
    {
        EntityWarship = new EntityWarship(speed, weight, bodyColor);
    }
    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawingWarshipWidth">Ширина прорисовки военного корабля</param>
    /// <param name="drawingWarshipHeight">Высота прорисовки военного корабля</param>
    protected DrawingWarship(int drawingWarshipWidth, int drawingWarshipHeight) : this()
    {
        _drawingWarshipWidth = drawingWarshipWidth;
        _pictureHeight = drawingWarshipHeight;
    }

    public DrawingWarship(EntityWarship warship)
    {
        EntityWarship = warship;
    }



    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя
    //разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        if (_drawingWarshipWidth < width && _drawingWarshipHeight < height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                SetPosition(_startPosX.Value, _startPosY.Value);
            }

            return true;
        }
        else
        {
            return false;
        }
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        if (x > 0 && y > 0 && x + _drawingWarshipWidth < _pictureWidth
            && y + _drawingWarshipHeight < _pictureHeight)
        {
            _startPosX = x;
            _startPosY = y;
        }
        else
        {
            Random rnd = new();
            _startPosX = rnd.Next(0, _pictureWidth.Value -
                _drawingWarshipWidth);
            _startPosY = rnd.Next(0, _pictureHeight.Value -
                _drawingWarshipHeight);
        }
    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение
    ///невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityWarship == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityWarship.Step > 0)
                {
                    _startPosX -= (int)EntityWarship.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityWarship.Step > 0)
                {
                    _startPosY -= (int)EntityWarship.Step;
                }
                return true;
            //вправо
            case DirectionType.Right:
                if (_startPosX + _drawingWarshipWidth + EntityWarship.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityWarship.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY + _drawingWarshipHeight + EntityWarship.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityWarship.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityWarship == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush bodyBrush = new SolidBrush(EntityWarship.BodyColor);
        //основная часть 
        Point[] body = new Point[] {new Point(_startPosX.Value + 5,
            _startPosY.Value), new Point(_startPosX.Value + 100,
            _startPosY.Value), new Point(_startPosX.Value + 150,
            _startPosY.Value + 25), new Point(_startPosX.Value + 100,
            _startPosY.Value + 50), new Point(_startPosX.Value + 5,
            _startPosY.Value + 50)};
        g.FillPolygon(bodyBrush, body);
        g.DrawPolygon(pen, body);
        Brush brBlack = new SolidBrush(Color.Black);
        g.FillRectangle(brBlack, _startPosX.Value,
            _startPosY.Value + 6, 5, 13);
        g.FillRectangle(brBlack, _startPosX.Value,
            _startPosY.Value + 31, 5, 13);
        Brush brDark = new SolidBrush(Color.DarkGray);
        g.FillRectangle(brDark, _startPosX.Value + 39,
            _startPosY.Value + 20, 40, 10);
        g.DrawRectangle(pen, _startPosX.Value + 39,
            _startPosY.Value + 20, 40, 10);
        g.FillRectangle(brDark, _startPosX.Value + 70,
            _startPosY.Value + 12, 18, 26);
        g.DrawRectangle(pen, _startPosX.Value + 70,
            _startPosY.Value + 12, 18, 26);
        g.FillEllipse(brBlack, _startPosX.Value + 94,
            _startPosY.Value + 19, 12, 12);
    }

    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;
    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;
    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawingWarshipWidth;
    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawingWarshipHeight;
}